(define-module (ordo cli run)
  #:use-module (config)
  #:use-module (config api)
  #:use-module (ice-9 filesystem)
  #:use-module (ordo inventory)
  #:use-module (ordo logger)
  #:use-module (ordo playbook)
  #:use-module (srfi srfi-1)
  #:use-module (srfi srfi-26)
  #:export (config handler))

(define (valid-tags? x)
  (or (null? x)
      (and (list? x) (every keyword? x))))

(define (parse-tags x)
  (map (compose symbol->keyword string->symbol)
       (if (list? x) x (list x))))

(define config
  (configuration
   (name 'run)
   (wanted '((keywords . (log-level)) directory))
   (keywords
    (list
     (setting
      (name 'inventory)
      (default "/dev/null")
      (example "examples/inventory.scm")
      (handler (cut expand-file-name <> #f #t))
      (test file-exists?)
      (synopsis "Inventory file"))
     (switch
      (name 'tag)
      (default (list))
      (test valid-tags?)
      (handler parse-tags)
      (merge-strategy cons)
      (synopsis "Limit operations to specified tag(s)"))))
   (arguments
    (list
     (argument
      (name 'playbook)
      (handler (cut expand-file-name <> #f #t))
      (test file-exists?))))
   (synopsis "Run a playbook")))

(define (handler options)
  (let ((inventory (load-inventory (option-ref options 'inventory)))
        (playbook (load-playbook (option-ref options '(playbook)))))
    (run-playbook playbook inventory)))
