(define-module (ordo connection)
  #:use-module (ice-9 exceptions)
  #:use-module (oop goops)
  #:use-module (ordo connection base)
  #:use-module (ordo connection local)
  #:use-module (ordo connection ssh)
  #:use-module (ordo connection sudo)
  #:use-module (ordo logger)
  #:use-module (ordo util flatten)
  #:use-module (ordo util keyword-args)
  #:use-module (srfi srfi-1)
  #:use-module (srfi srfi-71)
  #:export (connection?
            local-connection
            ssh-connection
            call-with-connection
            run)
  #:re-export (remote-exec with-remote-input-file with-remote-output-file))

(define (connection? c)
  (is-a? c <connection>))

(define (local-connection)
  (make <local-connection>))

(define* (ssh-connection host #:key (user (getlogin)) (password #f) (identity #f) (authenticate-server? #t))
  (make <ssh-connection> #:user user #:host host #:password password
        #:identity identity #:authenticate-server? authenticate-server?))

(define* (call-with-connection conn proc #:key sudo? sudo-user sudo-password)
  (let ((conn (deep-clone conn)))
    (when sudo?
      (unless (is-a? conn <sudo-connection>)
        (raise-exception
         (make-exception
          (make-programming-error)
          (make-exception-with-message (format #f "connection ~a does not support sudo" conn)))))
      (set! (become? conn) sudo?)
      (set! (become-user conn) sudo-user)
      (set! (become-password conn) sudo-password))
    (dynamic-wind
      (lambda () (setup conn))
      (lambda () (proc conn))
      (lambda () (teardown conn)))))

(define (run conn prog . args)
  (let* ((args options (break keyword? args))
         (args (remove unspecified? (flatten args)))
         (return (keyword-arg options #:return identity))
         (check? (keyword-arg options #:check?))
         (command (build-command conn prog args options)))
    (log-msg 'INFO "Running command: " command)
    (let ((out rc (remote-exec conn command)))
      (log-msg 'INFO "Command exit code: " rc)
      (if check?
          (if (zero? rc)
              (return out)
              (raise-exception (make-exception
                                (make-external-error)
                                (make-exception-with-message (format #f "Non-zero exit (~a) from ~a" rc prog)))))
          (values (return out) rc)))))
