(define-module (ordo connection base)
  #:use-module (ice-9 match)
  #:use-module (oop goops)
  #:use-module (ordo util flatten)
  #:use-module (ordo util keyword-args)
  #:use-module (ordo util shell-quote)
  #:use-module ((srfi srfi-1) #:select (remove))
  #:export (<connection>
            setup
            teardown
            build-command
            remote-exec
            with-remote-input-file
            with-remote-output-file))

(define-generic setup)
(define-generic teardown)
(define-generic build-command)
(define-generic remote-exec)
(define-generic with-remote-input-file)
(define-generic with-remote-output-file)

(define-class <connection> ())

(define-method (setup (c <connection>)) #t)

(define-method (teardown (c <connection>)) #t)

(define-method (build-command (c <connection>) (prog-name <string>) (prog-args <list>) (options <list>))
  (let* ((pwd (keyword-arg options #:pwd))
         (env (keyword-arg options #:env))
         (redirect-err? (keyword-arg options #:redirect-err?))
         (xs (remove unspecified?
                     (flatten (list "env"
                                    (when pwd (list "--chdir" (string-shell-quote pwd)))
                                    (when env (map (match-lambda ((k . v) (string-append k "=" (string-shell-quote v)))) env))
                                    prog-name
                                    (map string-shell-quote prog-args)
                                    (when redirect-err? "2>&1"))))))
    (string-join xs " ")))
