(define-module (ordo connection ssh)
  #:use-module (oop goops)
  #:use-module (ice-9 exceptions)
  #:use-module (ice-9 popen)
  #:use-module (ssh session)
  #:use-module (ssh channel)
  #:use-module (ssh auth)
  #:use-module (ssh popen)
  #:use-module (ssh sftp)
  #:use-module (ordo connection base)
  #:use-module (ordo connection sudo)
  #:use-module (ordo util read-lines)
  #:export (<ssh-connection>))

(define-class <ssh-connection> (<sudo-connection>)
  (user #:getter ssh-connection-user #:init-keyword #:user)
  (host #:getter ssh-connection-host #:init-keyword #:host)
  (password #:getter ssh-connection-password #:init-keyword #:password #:init-val #f)
  (identity #:getter ssh-connection-identity #:init-keyword #:identity #:init-val #f)
  (authenticate-server? #:getter ssh-connection-authenticate-server? #:init-keyword #:authenticate-server? #:init-val #t)
  (session)
  (sftp-session))

(define-method (setup (c <ssh-connection>))
  (unless (slot-bound? c 'session)
    (slot-set! c 'session (make-session #:user (ssh-connection-user c) #:host (ssh-connection-host c)))
    (when (ssh-connection-identity c) (session-set! (slot-ref c 'session) 'identity (ssh-connection-identity c))))
  (let ((s (slot-ref c 'session)))
    (unless (connected? s)
      (connect! s)
      (when (ssh-connection-authenticate-server? s)
        (let ((server-auth (authenticate-server s)))
          (unless (equal? 'ok server-auth)
            (error (format #f "authenticate-server: ~a" server-auth)))))
      (let ((user-auth (if (ssh-connection-password c)
                           (userauth-password! s (ssh-connection-password c))
                           (userauth-public-key/auto! s))))
        (unless (equal? 'success user-auth)
          (error (format #f "userauth: ~a" user-auth)))))))

(define-method (remote-exec (c <ssh-connection>) (command <string>))
  (let* ((channel (open-remote-input-pipe (slot-ref c 'session) command))
         (output (read-lines channel))
         (exit-status (channel-get-exit-status channel)))
    (close channel)
    (values output exit-status)))

(define-method (sftp-session (c <ssh-connection>))
  (unless (slot-bound? c 'sftp-session)
    (slot-set! c  'sftp-session (make-sftp-session (session c))))
  (slot-ref c 'sftp-session))

(define-method (with-remote-input-file (c <ssh-connection>) (filename <string>) (proc <procedure>))
  (call-with-remote-input-file (sftp-session c) filename proc))

(define-method (with-remote-output-file (c <ssh-connection>) (filename <string>) (proc <procedure>))
  (call-with-remote-output-file (sftp-session c) filename proc))

(define-method (teardown (c <ssh-connection>))
  (when (slot-bound? c 'session)
    (let ((s (slot-ref c session)))
      (when (connected? s)
        (disconnect! s)))))
