(define-module (ordo connection sudo)
  #:use-module (oop goops)
  #:use-module (ice-9 exceptions)
  #:use-module (ice-9 rdelim)
  #:use-module (srfi srfi-26)
  #:use-module (srfi srfi-71)
  #:use-module (ordo connection base)
  #:use-module (ordo util shell-quote)
  #:export (<sudo-connection>
            become?
            become-user
            become-password))

(define-class <sudo-connection> (<connection>)
  (become? #:accessor become? #:init-keyword become? #:init-form #f)
  (become-user #:accessor become-user #:init-keyword #:become-user #:init-form #f)
  (become-password #:accessor become-password #:init-keyword #:become-password #:init-form #f)
  (password-tmp-file #:accessor password-tmp-file))

(define-method (setup (conn <sudo-connection>))
  (when (become-password conn)
    (let ((out rc (exec conn "mktemp")))
      (unless (zero? rc)
        (raise-exception (make-exception
                          (make-external-error)
                          (make-exception-with-message (format #f "Failed to create temporary directory: ~a" (car out))))))
      (let ((tmp-file (car out)))
        (call-with-output-file conn tmp-file (cut write-line (become-password conn) <>))
        (set! (password-tmp-file conn) tmp-file)))))

(define-method (build-command (conn <sudo-connection>) (prog-name <string>) (prog-args <list>) (options <list>))
  (cond
   ((not (become? conn))
    (next-method))

   ((and (become-user conn) (become-password conn))
    (format #f "cat ~a - | sudo -k -S -H -u ~a -- ~a" (string-shell-quote (password-tmp-file conn)) (string-shell-quote (become-user conn)) (next-method)))

   ((become-password conn)
    (format #f "cat ~a - | sudo -k -S -H -- ~a" (string-shell-quote (password-tmp-file conn)) (next-method)))

   ((become-user conn)
    (format #f "sudo -k -n -H -u ~a -- ~a" (string-shell-quote (become-user conn)) (next-method)))

   (else (format #f "sudo -k -n -H -- ~a" (next-method)))))

(define-method (teardown (conn <sudo-connection>))
  (when (slot-bound? conn 'password-tmp-file)
    (exec conn (format #f "rm -f ~a" (string-shell-quote (password-tmp-file conn))))))
