(define-module (ordo play)
  #:use-module (oop goops)
  #:use-module (ordo connection)
  #:use-module (ordo context)
  #:use-module (ordo inventory)
  #:use-module (ordo logger)
  #:use-module (ordo util flatten)
  #:use-module (srfi srfi-26)
  #:use-module (srfi srfi-69)
  #:export (play
            play?
            play-host
            play-sudo?
            play-sudo-user
            play-sudo-password
            play-vars
            play-actions
            play-handlers
            run-play))

(define-class <play> ()
  (name #:init-keyword #:name #:getter play-name)
  (host #:init-keyword #:host #:getter play-host)
  (sudo? #:init-keyword #:sudo? #:getter play-sudo? #:init-value #f)
  (sudo-user #:init-keyword #:sudo-user #:getter play-sudo-user #:init-value #f)
  (sudo-password #:init-keyword #:sudo-password #:getter play-sudo-password #:init-value #f)
  (vars #:init-keyword #:vars #:getter  play-vars #:init-form (list))
  (actions #:init-keyword #:actions #:getter play-actions #:init-form (list))
  (handlers #:init-keyword #:handlers #:getter play-handlers #:init-form (list)))

(define-method (initialize (object <play>) initargs)
  (next-method)
  (slot-set! object 'vars (alist->hash-table (slot-ref object 'vars)))
  object)

(define (play . args)
  (apply make <play> args))

(define (run-play p)
  (log-msg 'NOTICE "Running play: " (play-name p))
  (parameterize ((*play-vars* (play-vars p)))
    (let ((hosts (resolve-hosts (*inventory*) (play-host p))))
      (if (null? hosts)
          (log-msg 'WARN "No hosts matched: " (play-host p))
          (for-each (lambda (h) (run-host-play p h)) hosts)))))

(define (run-host-play p h)
  (log-msg 'NOTICE "Running play: " (play-name p) " on host: " (host-name h))
  (parameterize ((*host-vars* (host-vars h)))
    (call-with-connection
     (host-connection h)
     (lambda (conn)
       #f
       )
     #:sudo? (play-sudo? p)
     #:sudo-user (play-sudo-user p)
     #:sudo-password (play-sudo-password p))))
